package org.freecompany.util.multicaster;

import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.logging.LogRecord;
import java.util.logging.MemoryHandler;

import static java.util.logging.Level.FINE;
import static java.util.logging.Level.INFO;
import static java.util.logging.Level.SEVERE;
import static java.util.logging.Level.WARNING;
import static java.util.logging.Logger.getLogger;

/**
 * Notification implementation that generates log messages describing
 * each notification event.  This notification system enhances the
 * flexibility of the core logging framework by allowing configurable
 * control to some expensive features of the built in logger, such as
 * calling class inferrence.
 */
public class LoggerNotification implements Notification {

	public static void enableBuffering() {
		final Logger root = getLogger( "");
		for ( Handler handler : root.getHandlers()) {
			final MemoryHandler buffer = new MemoryHandler( handler, 50, WARNING);
			root.removeHandler( handler);
			root.addHandler( buffer);
		}
	}

	protected final Logger logger;
	protected boolean infer = true;
	protected boolean trace = true;
	protected boolean mask = true;

	public LoggerNotification() {
		this( LoggerNotification.class.getName());
	}
	
	public LoggerNotification( final String name) {
		logger = getLogger( name);
	}
	
	public LoggerNotification( final boolean infer, final boolean trace, final boolean mask) {
		this( LoggerNotification.class.getName());
		this.infer = infer;
		this.trace = trace;
		this.mask = mask;
	}

	public LoggerNotification( final String name, final boolean infer, final boolean trace, final boolean mask) {
		this( name);
		this.infer = infer;
		this.trace = trace;
		this.mask = mask;
	}

	protected final void log( final Level level, final String message, final Throwable t, final Object... arguments) {
		if ( !logger.isLoggable( level)) return;
		logger.log( record( level, message, t, arguments));
	}

	protected final void log( final Level level, final String message, final String clazz, final String method, final Throwable t, final Object... arguments) {
		if ( !logger.isLoggable( level)) return;
		logger.log( record( level, message, clazz, method, t, arguments));
	}

	protected final LogRecord record( final Level level, String message, final Throwable t, final Object... arguments) {
		if ( message == null) {
			if ( arguments == null) message = "";
			else {
				final StringBuilder builder = new StringBuilder();
				for ( int x = 0; x < arguments.length; x++) {
					if ( x > 0) builder.append( ", ");
					builder.append( "{").append( x).append( "}");
				}
				message = builder.toString();
			}
		}

		String clazz = null, method = null;
		if ( infer) for ( StackTraceElement trace : new Throwable().getStackTrace()) {
			if ( !mask || !trace.getClassName().startsWith( "org.freecompany.util.multicaster")) {
				clazz = trace.getClassName();
				method = new StringBuilder( trace.getMethodName()).append( "#").append( trace.getLineNumber()).toString();
				break;
			}
		}
		return record( level, message, clazz, method, t, arguments);
	}

	protected final LogRecord record( final Level level, final String message, final String clazz, final String method, final Throwable t, final Object... arguments) {
		final LogRecord record = new LogRecord( level, message);
		if ( trace && t != null) record.setThrown( t);
		record.setSourceClassName( clazz);
		record.setSourceMethodName( method);
		record.setParameters( arguments);
		return record;
	}

	public void infer( final boolean infer) { this.infer = infer; }
	public void trace( final boolean trace) { this.trace = trace; }
	public void mask( final boolean mask) { this.mask = mask; }

	public final void debug( Throwable t, Object... arguments) { log( FINE, null, t, arguments); }
	public final void debug( String message, Object... arguments) { log( FINE, message, null, arguments); }
	public final void debug( String message, Throwable t, Object... arguments) { log( FINE, message, t, arguments); }

	public final void info( Throwable t, Object... arguments) { log( INFO, null, t, arguments); }
	public final void info( String message, Object... arguments) { log( INFO, message, null, arguments); }
	public final void info( String message, Throwable t, Object... arguments) { log( INFO, message, t, arguments); }

	public final void warning( Throwable t, Object... arguments) { log( WARNING, null, t, arguments); }
	public final void warning( String message, Object... arguments) { log( WARNING, message, null, arguments); }
	public final void warning( String message, Throwable t, Object... arguments) { log( WARNING, message, t, arguments); }

	public final void severe( Throwable t, Object... arguments) { log( SEVERE, null, t, arguments); }
	public final void severe( String message, Object... arguments) { log( SEVERE, message, null, arguments); }
	public final void severe( String message, Throwable t, Object... arguments) { log( SEVERE, message, t, arguments); }

	public final void fatal( Throwable t, Object... arguments) { log( SEVERE, null, t, arguments); }
	public final void fatal( String message, Object... arguments) { log( SEVERE, message, null, arguments); }
	public final void fatal( String message, Throwable t, Object... arguments) { log( SEVERE, message, t, arguments); }
}
